# 
# This MySQL script will create the appropriate squirrelmail
# database and tables. If you modify the user account, be sure to
# edit /etc/squirrelmail/config_local.php
# 

CREATE DATABASE IF NOT EXISTS squirrelmail;
GRANT ALL PRIVILEGES ON squirrelmail.* TO squirrelmail@localhost;

USE squirrelmail;

## User Preferences (SquirrelMail default)
CREATE TABLE IF NOT EXISTS userprefs (
  user varchar(128) DEFAULT '' NOT NULL,
  prefkey varchar(64) DEFAULT '' NOT NULL,
  prefval blob DEFAULT '' NOT NULL,
  PRIMARY KEY (user,prefkey)
);

## Address Book (SquirrelMail default)
CREATE TABLE IF NOT EXISTS address (
  owner varchar(128) DEFAULT '' NOT NULL,
  nickname varchar(16) DEFAULT '' NOT NULL,
  firstname varchar(128) DEFAULT '' NOT NULL,
  lastname varchar(128) DEFAULT '' NOT NULL,
  email varchar(128) DEFAULT '' NOT NULL,
  label varchar(255),
  PRIMARY KEY (owner,nickname),
  KEY firstname (firstname,lastname)
);
#######################################################################


## Begin For Calendar SQL Backend plugin

CREATE TABLE IF NOT EXISTS calendars (
  id varchar(250) NOT NULL,
  type varchar(30) NOT NULL,
  name varchar(255) default '',
  domain varchar(128) default '',
  created_on datetime,
  last_modified_on datetime,
  ical_raw text NOT NULL,
  PRIMARY KEY (id),  
  UNIQUE KEY id (id),
  KEY type (type)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS calendar_owners (
  calendar_id varchar(250) NOT NULL,
  owner_name varchar(128) NOT NULL,
  PRIMARY KEY (calendar_id, owner_name),  
  UNIQUE KEY cal_owner (calendar_id, owner_name)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS calendar_readers (
  calendar_id varchar(250) NOT NULL,
  reader_name varchar(128) NOT NULL,
  PRIMARY KEY (calendar_id, reader_name),  
  UNIQUE KEY cal_reader (calendar_id, reader_name)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS calendar_writers (
  calendar_id varchar(250) NOT NULL,
  writer_name varchar(128) NOT NULL,
  PRIMARY KEY (calendar_id, writer_name),  
  UNIQUE KEY cal_writer (calendar_id, writer_name)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS events (
  id int(11) NOT NULL AUTO_INCREMENT,
  event_id varchar(250) NOT NULL,
  calendar_id varchar(250) NOT NULL,
  domain varchar(128) default '',
  evt_start datetime,
  evt_end datetime,
  isAllDay enum('YES', 'NO') NOT NULL,
  isRecurring enum('YES', 'NO') NOT NULL,
  isTask enum('YES', 'NO') NOT NULL,
  isHoliday enum('YES', 'NO') NOT NULL,
  priority tinyint(3) unsigned default 5,  -- default NORMAL priority
  created_on datetime,
  last_modified_on datetime,
  ical_raw text NOT NULL,
  PRIMARY KEY (id),
  UNIQUE KEY id (id),
  UNIQUE KEY event_id (event_id, calendar_id),
  KEY isRecurring (isRecurring),
  KEY isTask (isTask),
  KEY isHoliday (isHoliday),
  KEY evt_start (evt_start),
  KEY evt_end (evt_end)        -- TODO: should create some sensible composite keys
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS event_parent_calendars (
  event_key int(11) NOT NULL,
  parent_calendar_id varchar(250) NOT NULL,
  PRIMARY KEY (event_key, parent_calendar_id),
  UNIQUE KEY event_parent (event_key, parent_calendar_id)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS event_owners (
  event_key int(11) NOT NULL,
  owner_name varchar(128) NOT NULL,
  PRIMARY KEY (event_key, owner_name),
  UNIQUE KEY event_owner (event_key, owner_name)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS event_readers (
  event_key int(11) NOT NULL,
  reader_name varchar(128) NOT NULL,
  PRIMARY KEY (event_key, reader_name),
  UNIQUE KEY event_reader (event_key, reader_name)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS event_writers (
  event_key int(11) NOT NULL,
  writer_name varchar(128) NOT NULL,
  PRIMARY KEY (event_key, writer_name),
  UNIQUE KEY event_writer (event_key, writer_name)
) TYPE=MyISAM;

## end Calendar SQL Backend
#######################################################################


## For squirrel_logger Plugin (disabled by default -- uses syslog) ####

#CREATE TABLE IF NOT EXISTS user_activity (
#  id int(11) NOT NULL auto_increment,
#  event varchar(30) NOT NULL,
#  username varchar(128) NOT NULL,
#  domain varchar(128) NOT NULL,
#  remote_address varchar(255) NOT NULL,
#  date datetime NOT NULL,
#  comments varchar(255) DEFAULT '',
#  PRIMARY KEY (id),
#  UNIQUE KEY id (id),
#  KEY event (event),
#  KEY domain (domain),
#  KEY date (date)
#) TYPE=MyISAM;
#######################################################################


## For vlogin Plugin (Advanced SQL Settings -- disabled by default) ###

#CREATE TABLE IF NOT EXISTS squirrelmail_settings (
#  username      varchar(128) default '',
#  domain        varchar(128) default '',
#  service_level varchar(128) default '',
#  setting_name  varchar(128) NOT NULL,
#  setting_value varchar(255) default '',
#  PRIMARY KEY (username, domain, service_level, setting_name),
#  UNIQUE KEY squirrelmailm_setting (username, domain, service_level, setting_name),
#  KEY username (username),
#  KEY domain (domain),
#  KEY service_level (service_level)
#) TYPE=MyISAM;
#
#######################################################################
